"use strict";
/*
* This program and the accompanying materials are made available under the terms of the
* Eclipse Public License v2.0 which accompanies this distribution, and is available at
* https://www.eclipse.org/legal/epl-v20.html
*
* SPDX-License-Identifier: EPL-2.0
*
* Copyright Contributors to the Zowe Project.
*
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.findNpmOnPath = findNpmOnPath;
exports.installPackages = installPackages;
exports.getRegistry = getRegistry;
exports.getScopeRegistry = getScopeRegistry;
exports.npmLogin = npmLogin;
exports.getPackageInfo = getPackageInfo;
const PMFConstants_1 = require("./PMFConstants");
const path = require("path");
const which = require("which");
const jsonfile_1 = require("jsonfile");
const npmPackageArg = require("npm-package-arg");
const pacote = require("pacote");
const utilities_1 = require("../../../../utilities");
const io_1 = require("../../../../io");
const npmCmd = findNpmOnPath();
/**
 * Common function that returns npm command as a string.
 *
 * @return {string} command with npm path
 */
function findNpmOnPath() {
    return which.sync("npm");
}
/**
 * Common function that installs a npm package using the local npm cli.
 * @param {string} prefix Path where to install npm the npm package.
 *
 * @param {string} registry The npm registry to install from.
 *
 * @param {string} npmPackage The name of package to install.
 *
 * @return {string} command response
 *
 */
function installPackages(prefix, registry, npmPackage) {
    const pipe = ["pipe", "pipe", process.stderr];
    const args = [
        "install", npmPackage,
        "--prefix", prefix,
        "-g"
    ];
    let isDirTest;
    try {
        isDirTest = io_1.IO.isDir(registry);
    }
    catch (e) {
        isDirTest = false;
    }
    if (!(registry.substring(registry.lastIndexOf(".") + 1) === "tgz") && !isDirTest) {
        args.push("--registry", registry);
    }
    args.push("--legacy-peer-deps");
    const execOutput = utilities_1.ExecUtils.spawnAndGetOutput(npmCmd, args, {
        cwd: PMFConstants_1.PMFConstants.instance.PMF_ROOT,
        stdio: pipe
    });
    return execOutput.toString();
}
/**
 * Get the registry to install to.
 *
 * @return {string}
 */
function getRegistry() {
    const execOutput = utilities_1.ExecUtils.spawnAndGetOutput(npmCmd, ["config", "get", "registry"]);
    return execOutput.toString();
}
function getScopeRegistry(scope) {
    const execOutput = utilities_1.ExecUtils.spawnAndGetOutput(npmCmd, ["config", "get", `@${scope}:registry`]);
    if (execOutput.toString().trim() === 'undefined')
        return getRegistry();
    return execOutput.toString();
}
/**
 * NPM login to be able to install from secure registry
 * @param {string} registry The npm registry to install from.
 */
function npmLogin(registry) {
    utilities_1.ExecUtils.spawnAndGetOutput(npmCmd, [
        "login",
        "--registry", registry,
        "--always-auth",
        "--auth-type=legacy"
    ], {
        stdio: [0, 1, 2]
    });
}
/**
 * Fetch name and version of NPM package that was installed
 * @param pkgSpec The package name as specified on NPM install
 */
function getPackageInfo(pkgSpec) {
    return __awaiter(this, void 0, void 0, function* () {
        const pkgInfo = npmPackageArg(pkgSpec);
        if (pkgInfo.registry) {
            // We already know package name, so read name and version from package.json
            return (0, jsonfile_1.readFileSync)(path.join(PMFConstants_1.PMFConstants.instance.PLUGIN_HOME_LOCATION, pkgInfo.name, "package.json"));
        }
        else {
            // Package name is unknown, so fetch name and version with pacote (npm SDK)
            return pacote.manifest(pkgSpec);
        }
    });
}
//# sourceMappingURL=NpmFunctions.js.map